-module(mup_server_sup).
-behaviour(supervisor).
-export([start_link/0, init/1, start_server/1, find_server/1, get_servers/0,
         reconfig/0]).
-include("mup.hrl").


start_link() ->
    supervisor:start_link({local, ?MODULE}, ?MODULE, []).

init(_) ->
    {ok, {{one_for_one, 3, 60000}, []}}.


start_server(Name) ->
    io:format("Starting server \"~w\"~n", [Name]),
    {ok, _Pid} = supervisor:start_child(?MODULE,
                                        {Name,
                                         {mup_server, start_link, [Name]},
                                         transient, 10000, worker,
                                         [mup_server]}).

find_server(#server{}=Server) ->
    find_server(Server#server.name);
find_server(Name) ->
    Children = supervisor:which_children(?MODULE),
    find_server(Name, Children).

find_server(Name, [{Name, Pid, _Type, _Modules}|_Rest]) when is_pid(Pid) ->
    {ok, Pid};
find_server(Name, [_|Rest]) ->
    find_server(Name, Rest);
find_server(_Name, []) ->
    undefined.


get_servers() ->
    get_servers(supervisor:which_children(?MODULE)).

get_servers([{Name, Pid, _Type, _Modules}|Rest]) when is_pid(Pid) ->
    [{Name, Pid}|get_servers(Rest)];
get_servers([_|Rest]) ->
    get_servers(Rest);
get_servers([]) ->
    [].


reconfig() ->
    WantedNames = mup_config:get_server_names(),
    reconfig(WantedNames),
    terminate_unwanted(get_servers(), WantedNames).

reconfig([Name|Rest]) ->
    case find_server(Name) of
        undefined ->
            start_server(Name);
        {ok, Pid} ->
            mup_server:reconfig(Pid)
    end,
    reconfig(Rest);
reconfig([]) ->
    ok.

terminate_unwanted([{Name, _Pid}|Rest], WantedNames) ->
    case lists:member(Name, WantedNames) of
        true ->
            ok;
        false ->
            io:format("Stopping server \"~w\"~n", [Name]),
            supervisor:terminate_child(?MODULE, Name),
            supervisor:delete_child(?MODULE, Name)
    end,
    terminate_unwanted(Rest, WantedNames);
terminate_unwanted([], _WantedNames) ->
    ok.
