-module(mup_util).
-export([option/2, option/3, is_application_loaded/1, join/2, sync_stop/1]).
-include_lib("eunit/include/eunit.hrl").


option(Key, [{Key, Value}|_]) ->
    Value;
option(Key, [Key|_]) ->
    true;
option(Key, [_|Options]) ->
    option(Key, Options);
option(_, []) ->
    undefined.

option(Key, Options, Default) ->
    case option(Key, Options) of
        undefined -> Default;
        Other -> Other
    end.

option_test_() ->
    [?_assertEqual(value2,
                   option(option2, [{option1, value1},
                                    {option2, value2},
                                    {option3, value3}])),
     ?_assertEqual(true,
                   option(option2, [option1, option2, option3])),
     ?_assertEqual(undefined,
                   option(option2, [{option1, value1}])),
     ?_assertEqual(undefined,
                   option(option2, [option1, option3])),
     ?_assertEqual(undefined,
                   option(option, [])),
     ?_assertEqual(default,
                   option(option, [], default)),
     ?_assertEqual(value,
                   option(option, [{option, value}], default))].




is_application_loaded(Name) ->
    is_application_loaded(Name, application:loaded_applications()).

is_application_loaded(Name, [{Name,_,_}|_]) ->
    true;
is_application_loaded(Name, [_|Loaded]) ->
    is_application_loaded(Name, Loaded);
is_application_loaded(_, []) ->
    false.


is_application_loaded_test_() ->
    [?_assertEqual(false, is_application_loaded(nonexistent)),
     ?_assertEqual(true, is_application_loaded(stdlib)),
     ?_assertEqual(true, is_application_loaded(kernel))].


join([], _Separator) ->
    [];
join([Value], _Separator) ->
    [Value];
join([Value|Rest], Separator) ->
    [Value, Separator|join(Rest, Separator)].
    

join_test_() ->
    [?_assertEqual([], join([], ",")),
     ?_assertEqual(["abc"], join(["abc"], ",")),
     ?_assertEqual(["abc", ",", "def", ",", "ghi"],
                   join(["abc", "def", "ghi"], ",")),
     ?_assertEqual([<<"abc">>, ",", <<"def">>, ",", <<"ghi">>],
                   join([<<"abc">>, <<"def">>, <<"ghi">>], ","))
    ].



sync_stop(Pid) ->
    MonitorRef = erlang:monitor(process, Pid),
    exit(Pid, shutdown),
    receive
       {'DOWN', MonitorRef, process, Pid, _Info} -> ok
    end.
